==> bootRes_1.0.tar.gz: bootRes/R/mdcc.R <==
mdcc <- function(chrono, clim, method = "response", start = -6, end =
                 9, timespan = NULL, vnames = NULL, sb = TRUE,
                 win.size = 25, win.offset = 1, startlast = TRUE,
                 boot = FALSE) {
	# TODO: status bar should correspond to whole window movement, not to single computations
        # startlast: gibt an, ob vom Ende (neueste Daten), oder vom
        # Anfang (älteste Daten) berechnet wird
  
  month.ids <- c(-1:-12, 1:12)
  errormsg1 <-
    "start and end have to define an interval in [-1, -2, ..., -12, 1, 2, ..., 12]."
  if (!is.element(start, month.ids) || !is.element(end, month.ids) ||
      which(month.ids == start) > which(month.ids == end)) {
                                        # check start and end dates
    stop(errormsg1)
  }
  ## climate data dispatcher gets called...
  clim <- climdispatch(clim)                    # properly formatted
                                        # climate data gets returned
                                        # here
  chrono.years <- as.numeric(row.names(chrono)) # get timespan of
                                        # chrono
  clim.years <- sort(unique(clim[, 1])) # get timespan of climate data
  if (chrono.years[1] <= clim.years[1]) {
    overlap <- na.omit(clim.years[match(chrono.years, clim.years)]) # get overlap of timespans
  } else {
    overlap <- na.omit(chrono.years[match(clim.years, chrono.years)])
  }
  if (is.null(timespan)) {
    start.year <- overlap[1]
    end.year <- tail(overlap, 1)
  } else {
    if (start > 0) {
      if (!is.element(timespan[1], overlap) ||
          !is.element(timespan[2], overlap)) {
        errormsg3 <- paste("timespan has to be between ", overlap[1],
                           " and ", tail(overlap, 1),
                           " for start dates in current year.",
                           sep = "")
        stop(errormsg3)
      } else {
        start.year <- timespan[1]
        end.year <- timespan[2]
      }
    } else {
      if (!is.element(timespan[1], overlap) ||
          !is.element(timespan[2], overlap)) {
        errormsg4 <- paste("timespan has to be between ", overlap[1] +
                           1, " and ", tail(overlap, 1),
                           " for start dates in previous year.", sep = "")
        stop(errormsg4)
      } else {
        start.year <- timespan[1]
        end.year <- timespan[2]
      }
    }
  }
  if (start < 0 && is.na(match((start.year - 1), clim.years))) { # check if a previous year is available in climatic data; otherwise set start.year + 1
    offset <- 1
  } else {
    offset <- 0
  }
  if (start < 0) { # make sure that data get truncated properly	
    interval.clim <-(start.year - 1 + offset):end.year
    interval.chrono <- (start.year + offset):end.year
  } else {
    interval.clim <-(start.year + offset):end.year
    interval.chrono <- (start.year + 1 + offset):end.year
  }
  a <- as.numeric(rownames(chrono)) %in% interval.chrono
  b <- clim[, 1] %in% interval.clim
  chrono.trunc <- chrono[a, 1] # truncate data 
  clim.trunc <- clim[b, ]
  p <- pmat(clim.trunc, start, end, vnames) # call pmat to obtain
                                        # variable matrix
  ## number of windows
  years <- as.numeric(rownames(p))
  n.years <- length(years)
  win.num <- (length(chrono.trunc) - win.size) %/% win.offset
  win.years.string <- character(win.num)
  windows <- 1:win.num
  ## result matrix
  result.matrix <- matrix(NA, ncol = win.num, nrow = dim(p)[2])

  if (sb) { # initialize status bar (if TRUE)
    require(utils)
    mpb <- txtProgressBar(min = 1,  max = win.num, style = 3)
  } 

  if (startlast) {
    for (k in 1:win.num) {
      series.subset.index <- ((n.years - ((k-1)*win.offset)) -
                              (win.size - 1)):(n.years - ((k-1)*win.offset))
      p.win <- p[series.subset.index,]
      chrono.win <- chrono.trunc[series.subset.index]
      METHOD <- match.arg(method, c("response", "correlation"))  # match method argument
      if (METHOD == "response") {
        if (boot) {
          dc.coef <- brf(chrono.win, p.win, sb = FALSE, vnames = vnames)$coef # call brf to
                                        # calculate bootstrapped
                                        # response function
        } else {
          dc.coef <- nbrf(chrono.win, p.win, vnames = vnames)$coef # call rf for
                                        # non-bootstrapped response
                                        # function
        }
      }
      if (METHOD == "correlation") {
        if (boot) {
          dc.coef <- bcf(chrono.win, p.win, sb = FALSE, vnames = vnames)$coef # call bcf to
                                        # calculate bootstrapped
                                        # correlation function
        } else {
          dc.coef <- nbcf(chrono.win, p.win, vnames = vnames)$coef # call cf for
                                        # non-bootstrapped correlation
                                        # function
        }
      }
      result.matrix[,k] <- dc.coef
      win.years.string[k] <- paste(years[series.subset.index][1],
                                   years[series.subset.index][win.size],
                                   sep = "-")
      if (sb) # update status bar (if TRUE)
        setTxtProgressBar(mpb, k)
    }
    result.matrix <- result.matrix[,win.num:1]
    win.years.string <- win.years.string[win.num:1]
  } else {
    for (k in 1:win.num) {
      series.subset.index <- (1 + ((k-1) * win.offset)):(1 + ((k-1) *
                                                              win.offset) +
                                                         (win.size - 1))
      p.win <- p[series.subset.index,]
      chrono.win <- chrono.trunc[series.subset.index]
      METHOD <- match.arg(method, c("response", "correlation"))  # match method argument
      if (METHOD == "response") {
        if (boot) {
          dc.coef <- brf(chrono.win, p.win, sb = FALSE, vnames = vnames)$coef # call brf to
                                        # calculate bootstrapped
                                        # response function
        } else {
          dc.coef <- nbrf(chrono.win, p.win, vnames = vnames)$coef # call rf for
                                        # non-bootstrapped response
                                        # function
        }
      }
      if (METHOD == "correlation") {
        if (boot) {
          dc.coef <- bcf(chrono.win, p.win, sb = FALSE, vnames = vnames)$coef # call bcf to
                                        # calculate bootstrapped
                                        # correlation function
        } else {
          dc.coef <- nbcf(chrono.win, p.win, vnames = vnames)$coef # call cf for
                                        # non-bootstrapped correlation
                                        # function
        }
      }
      result.matrix[,k] <- dc.coef
      win.years.string[k] <- paste(years[series.subset.index][1],
                                   years[series.subset.index][win.size],
                                   sep = "-")
      if (sb) # update status bar (if TRUE)
        setTxtProgressBar(mpb, k)
    }
  }
  dc <- result.matrix
  colnames(dc) <- win.years.string
  rownames(dc) <- colnames(p.win)
  if (sb) # close status bar (if TRUE)
    close(mpb)
  dc
}

==> bootRes_1.1.tar.gz: bootRes/R/mdcc.R <==
mdcc <- function(chrono, clim, method = "response", start = -6, end =
                 9, timespan = NULL, vnames = NULL, sb = TRUE,
                 win.size = 25, win.offset = 1, startlast = TRUE,
                 boot = FALSE, ci = 0.05) {
	# TODO: status bar should correspond to whole window movement, not to single computations
        # startlast: gibt an, ob vom Ende (neueste Daten), oder vom
        # Anfang (älteste Daten) berechnet wird
  
  month.ids <- c(-1:-12, 1:12)
  errormsg1 <-
    "start and end have to define an interval in [-1, -2, ..., -12, 1, 2, ..., 12]."
  if (!is.element(start, month.ids) || !is.element(end, month.ids) ||
      which(month.ids == start) > which(month.ids == end)) {
                                        # check start and end dates
    stop(errormsg1)
  }
  ## climate data dispatcher gets called...
  clim <- climdispatch(clim)                    # properly formatted
                                        # climate data gets returned
                                        # here
  chrono.years <- as.numeric(row.names(chrono)) # get timespan of
                                        # chrono
  clim.years <- sort(unique(clim[, 1])) # get timespan of climate data
  if (chrono.years[1] <= clim.years[1]) {
    overlap <- na.omit(clim.years[match(chrono.years, clim.years)]) # get overlap of timespans
  } else {
    overlap <- na.omit(chrono.years[match(clim.years, chrono.years)])
  }
  if (is.null(timespan)) {
    start.year <- overlap[1]
    end.year <- tail(overlap, 1)
  } else {
    if (start > 0) {
      if (!is.element(timespan[1], overlap) ||
          !is.element(timespan[2], overlap)) {
        errormsg3 <- paste("timespan has to be between ", overlap[1],
                           " and ", tail(overlap, 1),
                           " for start dates in current year.",
                           sep = "")
        stop(errormsg3)
      } else {
        start.year <- timespan[1]
        end.year <- timespan[2]
      }
    } else {
      if (!is.element(timespan[1], overlap) ||
          !is.element(timespan[2], overlap)) {
        errormsg4 <- paste("timespan has to be between ", overlap[1] +
                           1, " and ", tail(overlap, 1),
                           " for start dates in previous year.", sep = "")
        stop(errormsg4)
      } else {
        start.year <- timespan[1]
        end.year <- timespan[2]
      }
    }
  }
  if (start < 0 && is.na(match((start.year - 1), clim.years))) { # check if a previous year is available in climatic data; otherwise set start.year + 1
    offset <- 1
  } else {
    offset <- 0
  }
  if (start < 0) { # make sure that data get truncated properly	
    interval.clim <-(start.year - 1 + offset):end.year
    interval.chrono <- (start.year + offset):end.year
  } else {
    interval.clim <-(start.year + offset):end.year
    interval.chrono <- (start.year + 1 + offset):end.year
  }
  a <- as.numeric(rownames(chrono)) %in% interval.chrono
  b <- clim[, 1] %in% interval.clim
  chrono.trunc <- chrono[a, 1] # truncate data 
  clim.trunc <- clim[b, ]
  p <- pmat(clim.trunc, start, end, vnames) # call pmat to obtain
                                        # variable matrix
  ## number of windows
  years <- as.numeric(rownames(p))
  n.years <- length(years)
  win.num <- (length(chrono.trunc) - win.size) %/% win.offset
  win.years.string <- character(win.num)
  windows <- 1:win.num
  ## result matrix
  result.matrix <- matrix(NA, ncol = win.num, nrow = dim(p)[2])

  if (sb) { # initialize status bar (if TRUE)
    mpb <- txtProgressBar(min = 1,  max = win.num, style = 3)
  } 

  if (startlast) {
    for (k in 1:win.num) {
      series.subset.index <- ((n.years - ((k-1)*win.offset)) -
                              (win.size - 1)):(n.years - ((k-1)*win.offset))
      p.win <- p[series.subset.index,]
      chrono.win <- chrono.trunc[series.subset.index]
      METHOD <- match.arg(method, c("response", "correlation"))  # match method argument
      if (METHOD == "response") {
        if (boot) {
          dc.coef <- brf(chrono.win, p.win, sb = FALSE, vnames =
                         vnames, ci = ci)$coef # call brf to
                                        # calculate bootstrapped
                                        # response function
        } else {
          dc.coef <- nbrf(chrono.win, p.win, vnames = vnames)$coef # call rf for
                                        # non-bootstrapped response
                                        # function
        }
      }
      if (METHOD == "correlation") {
        if (boot) {
          dc.coef <- bcf(chrono.win, p.win, sb = FALSE, vnames =
                         vnames, ci = ci)$coef # call bcf to
                                        # calculate bootstrapped
                                        # correlation function
        } else {
          dc.coef <- nbcf(chrono.win, p.win, vnames = vnames)$coef # call cf for
                                        # non-bootstrapped correlation
                                        # function
        }
      }
      result.matrix[,k] <- dc.coef
      win.years.string[k] <- paste(years[series.subset.index][1],
                                   years[series.subset.index][win.size],
                                   sep = "-")
      if (sb) # update status bar (if TRUE)
        setTxtProgressBar(mpb, k)
    }
    result.matrix <- result.matrix[,win.num:1]
    win.years.string <- win.years.string[win.num:1]
  } else {
    for (k in 1:win.num) {
      series.subset.index <- (1 + ((k-1) * win.offset)):(1 + ((k-1) *
                                                              win.offset) +
                                                         (win.size - 1))
      p.win <- p[series.subset.index,]
      chrono.win <- chrono.trunc[series.subset.index]
      METHOD <- match.arg(method, c("response", "correlation"))  # match method argument
      if (METHOD == "response") {
        if (boot) {
          dc.coef <- brf(chrono.win, p.win, sb = FALSE, vnames =
                         vnames, ci = ci)$coef # call brf to
                                        # calculate bootstrapped
                                        # response function
        } else {
          dc.coef <- nbrf(chrono.win, p.win, vnames = vnames)$coef # call rf for
                                        # non-bootstrapped response
                                        # function
        }
      }
      if (METHOD == "correlation") {
        if (boot) {
          dc.coef <- bcf(chrono.win, p.win, sb = FALSE, vnames =
                         vnames, ci = ci)$coef # call bcf to
                                        # calculate bootstrapped
                                        # correlation function
        } else {
          dc.coef <- nbcf(chrono.win, p.win, vnames = vnames)$coef # call cf for
                                        # non-bootstrapped correlation
                                        # function
        }
      }
      result.matrix[,k] <- dc.coef
      win.years.string[k] <- paste(years[series.subset.index][1],
                                   years[series.subset.index][win.size],
                                   sep = "-")
      if (sb) # update status bar (if TRUE)
        setTxtProgressBar(mpb, k)
    }
  }
  dc <- result.matrix
  colnames(dc) <- win.years.string
  rownames(dc) <- colnames(p.win)
  if (sb) # close status bar (if TRUE)
    close(mpb)
  dc
}
